@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Dang Mai <contact@dangmai.net>.
+# Copyright 2016, Dang Mai <contact@dangmai.net>.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -16,16 +16,17 @@
 """Generates smart playlists based on beets queries.
 """
 
-from __future__ import (division, absolute_import, print_function,
-                        unicode_literals)
+from __future__ import division, absolute_import, print_function
 
 from beets.plugins import BeetsPlugin
 from beets import ui
-from beets.util import mkdirall, normpath, syspath
+from beets.util import (mkdirall, normpath, sanitize_path, syspath,
+                        bytestring_path)
 from beets.library import Item, Album, parse_query_string
 from beets.dbcore import OrQuery
 from beets.dbcore.query import MultipleSort, ParsingError
 import os
+import six
 
 
 class SmartPlaylistPlugin(BeetsPlugin):
@@ -46,9 +47,11 @@ class SmartPlaylistPlugin(BeetsPlugin):
             self.register_listener('database_change', self.db_change)
 
     def commands(self):
-        spl_update = ui.Subcommand('splupdate',
-                                   help='update the smart playlists. Playlist '
-                                        'names may be passed as arguments.')
+        spl_update = ui.Subcommand(
+            'splupdate',
+            help=u'update the smart playlists. Playlist names may be '
+            u'passed as arguments.'
+        )
         spl_update.func = self.update_cmd
         return [spl_update]
 
@@ -64,9 +67,10 @@ class SmartPlaylistPlugin(BeetsPlugin):
                             for name, q, a_q in self._unmatched_playlists
                             if name in args)
             if not playlists:
-                raise ui.UserError('No playlist matching any of {0} '
-                                   'found'.format([name for name, _, _ in
-                                                   self._unmatched_playlists]))
+                raise ui.UserError(
+                    u'No playlist matching any of {0} found'.format(
+                        [name for name, _, _ in self._unmatched_playlists])
+                )
 
             self._matched_playlists = playlists
             self._unmatched_playlists -= playlists
@@ -95,7 +99,7 @@ class SmartPlaylistPlugin(BeetsPlugin):
 
         for playlist in self.config['playlists'].get(list):
             if 'name' not in playlist:
-                self._log.warn("playlist configuration is missing name")
+                self._log.warning(u"playlist configuration is missing name")
                 continue
 
             playlist_data = (playlist['name'],)
@@ -104,7 +108,7 @@ class SmartPlaylistPlugin(BeetsPlugin):
                     qs = playlist.get(key)
                     if qs is None:
                         query_and_sort = None, None
-                    elif isinstance(qs, basestring):
+                    elif isinstance(qs, six.string_types):
                         query_and_sort = parse_query_string(qs, Model)
                     elif len(qs) == 1:
                         query_and_sort = parse_query_string(qs[0], Model)
@@ -131,8 +135,8 @@ class SmartPlaylistPlugin(BeetsPlugin):
                     playlist_data += (query_and_sort,)
 
             except ParsingError as exc:
-                self._log.warn("invalid query in playlist {}: {}",
-                               playlist['name'], exc)
+                self._log.warning(u"invalid query in playlist {}: {}",
+                                  playlist['name'], exc)
                 continue
 
             self._unmatched_playlists.add(playlist_data)
@@ -151,17 +155,19 @@ class SmartPlaylistPlugin(BeetsPlugin):
         for playlist in self._unmatched_playlists:
             n, (q, _), (a_q, _) = playlist
             if self.matches(model, q, a_q):
-                self._log.debug("{0} will be updated because of {1}", n, model)
+                self._log.debug(
+                    u"{0} will be updated because of {1}", n, model)
                 self._matched_playlists.add(playlist)
                 self.register_listener('cli_exit', self.update_playlists)
 
         self._unmatched_playlists -= self._matched_playlists
 
     def update_playlists(self, lib):
-        self._log.info("Updating {0} smart playlists...",
+        self._log.info(u"Updating {0} smart playlists...",
                        len(self._matched_playlists))
 
         playlist_dir = self.config['playlist_dir'].as_filename()
+        playlist_dir = bytestring_path(playlist_dir)
         relative_to = self.config['relative_to'].get()
         if relative_to:
             relative_to = normpath(relative_to)
@@ -182,6 +188,7 @@ class SmartPlaylistPlugin(BeetsPlugin):
             # the items and generate the correct m3u file names.
             for item in items:
                 m3u_name = item.evaluate_template(name, True)
+                m3u_name = sanitize_path(m3u_name, lib.replacements)
                 if m3u_name not in m3us:
                     m3us[m3u_name] = []
                 item_path = item.path
@@ -191,9 +198,11 @@ class SmartPlaylistPlugin(BeetsPlugin):
                     m3us[m3u_name].append(item_path)
             # Now iterate through the m3us that we need to generate
             for m3u in m3us:
-                m3u_path = normpath(os.path.join(playlist_dir, m3u))
+                m3u_path = normpath(os.path.join(playlist_dir,
+                                    bytestring_path(m3u)))
                 mkdirall(m3u_path)
-                with open(syspath(m3u_path), 'w') as f:
+                with open(syspath(m3u_path), 'wb') as f:
                     for path in m3us[m3u]:
                         f.write(path + b'\n')
-        self._log.info("{0} playlists updated", len(self._matched_playlists))
+
+        self._log.info(u"{0} playlists updated", len(self._matched_playlists))
